/* PPLite: a lightweight library for convex polyhedra derived from PPL.
   Copyright (C) 2018 Enea Zaffanella <enea.zaffanella@unipr.it>

This file is part of PPLite.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "pplite_test.hh"

bool
test01() {
  Var A(0);
  Var B(1);

  Poly ph1(2);
  ph1.add_con(A == 0);
  ph1.add_con(B == 1);

  print_cons(ph1, "*** ph1 ***");

  Poly ph2(2);
  ph2.add_con(A == 0);
  ph2.add_con(B == 4);

  print_cons(ph2, "*** ph2 ***");

  ph1.con_hull_assign(ph2);

  Poly kr(2);
  kr.add_con(A == 0);
  kr.add_con(B >= 1);
  kr.add_con(B <= 4);

  print_cons(ph1, "*** ph1.con_hull_assign(ph2) ***");
  print_cons(kr, "*** kr ***");

  return ph1 == kr;
}

bool
test02() {
  Var A(0);
  Var B(1);

  Poly ph1(2);
  ph1.add_con(A == 0);
  ph1.add_con(B == 1);

  print_cons(ph1, "*** ph1 ***");

  Poly ph2(2);
  ph2.add_con(A == -2);
  ph2.add_con(B == 4);

  print_cons(ph2, "*** ph2 ***");

  ph1.con_hull_assign(ph2);

  Poly kr(2);
  kr.add_con(A >= -2);
  kr.add_con(A <= 0);
  kr.add_con(B >= 1);
  kr.add_con(B <= 4);

  print_cons(ph1, "*** ph1.con_hull_assign(ph2) ***");
  print_cons(kr, "*** kr ***");

  return ph1 == kr;
}

BEGIN_MAIN
  DO_TEST(test01);
  DO_TEST(test02);
END_MAIN
